/*
   Copyright 2010 Alfredo Braunstein, Farbod Kayhan and Riccardo Zecchina

   This file is a part of GFRBP, a lossy compressor of random independent
   binary sources in GF(q).

   This is a free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This software is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this file; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef CAVITY_H
#define CAVITY_H

#include <vector>
#include <iostream>

template<typename T, typename C = T (*) (T const &, T const &)>
class Cavity : public std::vector<T> {
public:


	template<class container_t>
	Cavity(container_t const & c, T const & init, C const & convolute)
	{
		  initialize(c.begin(), c.end(), init, convolute);
	}

	template<class iterator_t>
	Cavity(iterator_t const & beg, iterator_t const & end, T const & init, C const & convolute)
	{
		initialize(beg, end, init, convolute);
	}
	template<class iterator_t>
	void initialize(iterator_t const & beg, iterator_t const & end, T const & init, C const & convolute)
	{
		std::vector<T>::resize(distance(beg, end), init);
		int const n = std::vector<T>::size();
		if (n == 0) {
			full_ = init;
			return;
		}

		if (n == 1) {
			std::vector<T>::operator[](0) = init;
			full_ = *beg;
			return;
		}

		std::vector<T> right(n);
		iterator_t beg2 = beg, end2 = end;
		--end2;
		right[n - 1] = *end2;
		--end2;
		for (int i = n - 2; i >= 0; --i, --end2)
			right[i] = convolute(*end2, right[i + 1]);
		full_ = right[0];

		std::vector<T>::operator[](0) = right[1];
		T left = *beg2;
		for (int i = 1; i < n - 1; ++i) {
			std::vector<T>::operator[](i) = convolute(left, right[i + 1]);
			//std::cout << left.size() << " " << (beg2 + 1)->size() << "  " << i << " " << n << std::endl;
			//assert(left.size() == (beg2 + 1)->size());
			left = convolute(left, *(++beg2));
		}
		std::vector<T>::operator[](n - 1) = left;
	}

	T const & full() const { return full_; }

private:
	T full_;
};

#endif
