/*
   Copyright 2010 Alfredo Braunstein, Farbod Kayhan and Riccardo Zecchina

   This file is a part of GFRBP, a lossy compressor of random independent
   binary sources in GF(q).

   This is a free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This software is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this file; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#include "formula.hpp"
#include "gfrbp.hpp"

#include <iostream>
#include <vector>
#include <cstdlib>
#include <cmath>

using namespace std;

//----------------------------------------------------------------------------
//global variables and structures:

int N;                              //Number of variables
int M;                              //Number of Checknodes
int Q;                              //Finite Field Order
int q;                              //Log2 of Q,
int l_max = 1000;                   //Max number of iterations
double epsilon = 1.0e-20;           //The precision for convergence
double gamma1 = 0.00001;                  //The reinforcement parameter gamma_1
double gamma0 = 0.00;                //The reinforcement parameter gamma_0
double L = 2.6;                     //The prior (external Field)
double gamma_fun;                      //The Reinforcement function
//char const * filename = "codes/FG_Q64_N1000_M500_s0_r5.txt";
//char const * filename = "codes/FG_Q64_N2000_M1000_s0_r5_b.txt";
char const * filename = "/dev/stdin";
//char const * filename = "codes/peg_2000_256_r5_labels.txt";
vector<varstruct> variables;        //Structure defined in main.h
vector<clausestruct> clauses;       //structure defined in main.h
extern int conv_itnum;
//end of clarifying the variables
//----------------------------------------------------------------------

int main()
{
	srand(2);
	int simul_num = 30;

	real_t av_dist = 0;
	real_t std_dist = 0;
	real_t av_it = 0;
	real_t std_it = 0;

	//Reading the corresponding FG from a file:formula.cpp
	readformula(filename);

	// leaf removal check
	leaf_removal();

	for (int i = 0; i < simul_num; ++i) {
            	//Random sequence, to be Compressed:formula.cpp
		createseq();		
		real_t mindist = 1;
			//Initialization of messages and external field:formula.cpp
		initialmemo();          		
		//RBP over GF(q):gfrbp.cpp
		real_t dist = gfreinforced();
		//real_t energy = energy_cal();
		if (dist < mindist)
			mindist = dist;
		//Writing the results into a file:gfrbp.cpp
		//writesolution(simul_num);
		cout << "distortion: " << mindist << endl;
		av_dist += mindist;
		std_dist += mindist * mindist;
		av_it += conv_itnum;
		std_it += conv_itnum * conv_itnum;
	}

	std_dist = sqrt((std_dist - av_dist/simul_num * av_dist))/simul_num;
	av_dist /= simul_num;
	std_it = sqrt(std_it - av_it / simul_num * av_it)/simul_num;
	av_it /= simul_num;
	cout << gamma0 << " " << gamma1 << " " << L << " " << av_dist << "±" << std_dist << " " << av_it << "±" << std_it << " ( " << simul_num << " )" << endl;

	return 0;
}
